//
//  GMTDDeliveryVehicleReporter.h
//  Google Ridesharing SDK for iOS
//
//  Copyright 2021 Google LLC
//
//  Usage of this SDK is subject to the Google Maps/Google Earth APIs Terms of
//  Service: https://cloud.google.com/maps-platform/terms
//

#import "GMTDVehicleReporter.h"

@class GMTDVehicleStop;

NS_ASSUME_NONNULL_BEGIN

/**
 * Object for sending information to the Fleet Engine backend. Used for deliveries.
 */
@interface GMTDDeliveryVehicleReporter : GMTDVehicleReporter

/**
 * \defgroup GMTDVehicleReporterStopCompletionHandler GMTDVehicleReporterStopCompletionHandler
 * @{
 */

/**
 * Handler for updating stops via the @c GMTDDeliveryVehicleReporter.
 *
 * @param stops The updated stops.
 * @param error The error that occurs if any.
 */
typedef void (^GMTDVehicleReporterStopCompletionHandler)(
    NSArray<GMTDVehicleStop *> *_Nullable stops, NSError *_Nullable error);

/**@}*/

/**
 * Updates the remaining VehicleStops for the vehicle. Results are reflected asynchronously in
 * @c remainingVehicleStops. The vehicle is expected to visit the stops in the same order as the
 * array.
 *
 * @param vehicleStops The ordered array of @c GMTDVehicleStop objects the vehicle will visit. This
 * must be non-nil and all stops other than the first stop must have a state of
 * @c GMTDVehicleStopStateNew.
 * @param completion A block to run after the stops have been updated.
 */
- (void)setVehicleStops:(NSArray<GMTDVehicleStop *> *)vehicleStops
             completion:(GMTDVehicleReporterStopCompletionHandler)completion;

/**
 * Updates the state of the first @c GMTDVehicleStop in @c remainingVehicleStops
 * to @c GMTDVehicleStopStateEnroute. @c remainingVehicleStops must contain at least one stop when
 * this function is called.
 *
 * @param completion A block to run after the stops have been updated.
 */
- (void)reportEnrouteToNextStopWithCompletion:(GMTDVehicleReporterStopCompletionHandler)completion;

/**
 * Updates the state of the first @c GMTDVehicleStop in @c remainingVehicleStops to @c
 * GMTDVehicleStopStateArrived. @c remainingVehicleStops must contain at least one stop when this
 * function is called.
 *
 *  @param completion A block to run after the stops have been updated.
 */
- (void)reportArrivedAtStopWithCompletion:(GMTDVehicleReporterStopCompletionHandler)completion;

/**
 * Removes the first @c GMTDVehicleStop in @c remainingVehicleStops. @c remainingVehicleStops must
 * contain at least one stop when this function is called.
 *
 * @param completion A block to run after the stops have been updated.
 */
- (void)reportCompletedStopWithCompletion:(GMTDVehicleReporterStopCompletionHandler)completion;

/**
 * Gets the remaining @c GMTDVehicleStop objects that the vehicle still needs to visit.
 *
 * @param completion A completion to run with the retrieved @c GMTDVehicleStop objects.
 *                   The completion will be run asynchronously on the main thread.
 */
- (void)getRemainingVehicleStopsWithCompletion:(GMTDVehicleReporterStopCompletionHandler)completion;

@end

NS_ASSUME_NONNULL_END
