//
//  GMTDVehicleReporter.h
//  Google Ridesharing SDK for iOS
//
//  Copyright 2021 Google LLC
//
//  Usage of this SDK is subject to the Google Maps/Google Earth APIs Terms of
//  Service: https://cloud.google.com/maps-platform/terms
//

#import <Foundation/Foundation.h>
#import <GoogleNavigation/GoogleNavigation.h>

#import "GMTDFleetEngineEnums.h"

@class GMTDServices;
@class GMTDVehicleReporter;
@class GMTDVehicleUpdate;

NS_ASSUME_NONNULL_BEGIN

/** The error domain for @c GMTDVehicleReporter. */
FOUNDATION_EXTERN NSErrorDomain const GMTDVehicleReporterErrorDomain;

/**
 * \defgroup GMTDVehicleReporterErrorCode GMTDVehicleReporterErrorCode
 * @{
 */

/** The error codes for @c GMTDVehicleReporter. */
typedef NS_ENUM(NSInteger, GMTDVehicleReporterErrorCode) {
  GMTDVehicleReporterErrorGeneral = 1,
  GMTDVehicleReporterErrorNoActiveTrips,
  GMTDVehicleReporterErrorActiveTripsFetchFailed,
  GMTDVehicleReporterErrorLocationTrackingNotEnabled,
  GMTDVehicleReporterErrorNoLastKnownLocation,
};

/**@}*/

/** Protocol for responding to periodic vehicle updates sent by @c GMTDVehicleReporter. */
@protocol GMTDVehicleReporterListener <NSObject>

/**
 * Informs the listeners that the Fleet Engine backend successfully received the vehicle location
 * and state update.
 *
 * @param vehicleReporter Indicates the instance of @c GMTDVehicleReporter that generated the event.
 * @param vehicleUpdate Contains the current location and state of the vehicle.
 */
- (void)vehicleReporter:(GMTDVehicleReporter *)vehicleReporter
    didSucceedVehicleUpdate:(GMTDVehicleUpdate *)vehicleUpdate;

/**
 * Informs the listeners that a vehicle update failed. As long as location tracking is enabled,
 * @c GMTDVehicleReporter continues to send the latest data to the Fleet Engine backend.
 *
 * If location tracking is disabled and this is for updating the vehicle state to OFFLINE, the
 * developer can call @c updateVehicleState: with @c GMTDVehicleStateOffline to trigger one-off
 * updates.
 *
 * @param vehicleReporter Indicates the instance of @c GMTDVehicleReporter that generated the event.
 * @param vehicleUpdate Contains the current location and state of the vehicle.
 * @param error Indicates the reason that an error occurred.
 */
- (void)vehicleReporter:(GMTDVehicleReporter *)vehicleReporter
    didFailVehicleUpdate:(GMTDVehicleUpdate *)vehicleUpdate
               withError:(NSError *)error;

@end

/**
 * Object for sending information to the Fleet Engine backend.
 *
 * Use this class only from the main thread.
 */
@interface GMTDVehicleReporter
    : NSObject <GMSNavigatorListener, GMSRoadSnappedLocationProviderListener>

/**
 * This class has no public initializers; obtain this object from the @c vehicleReporter property
 * of the @c GMTDDeliveryDriverAPI or @c GMTDRidesharingDriverAPI object.
 */
- (null_unspecified instancetype)init NS_UNAVAILABLE;

/**
 * Indicates whether location tracking is enabled.
 *
 * If set to YES, trip and vehicle updates are sent to the Fleet Engine backend at a regular
 * interval based on the value set for @c locationUpdateInterval.
 *
 * If set to NO, updates stop and a one-off vehicle update request is sent to the Fleet Engine
 * backend to set the vehicle state to @c GMTDVehicleStateOffline. See @c updateVehicleState
 * for special considerations on handling failures when @c locationTrackingEnabled is set to NO.
 */
@property(nonatomic) BOOL locationTrackingEnabled;

/**
 * Indicates the minimum interval at which location reports will be delivered to Fleet Engine.
 *
 * NOTE: The default reporting interval is 10 seconds. The maximum supported value is 60 seconds
 * and the minimum supported value is 5 seconds. If a value outside of this range is used, the
 * requested value is clamped to that range.
 */
@property(nonatomic) NSTimeInterval locationReportingInterval;

/**
 * Updates the vehicle state. Used only for ridesharing.
 *
 * If @c locationTrackingEnabled is set to NO, setting the state to @c GMTDVehicleStateOnline fails.
 *
 * Setting a different state will send a one-off request to the Fleet Engine backend.
 *
 * This method updates a server-side state, the client app should monitor
 * @c fleetEngine:didSucceedVehicleUpdate: and @c fleetEngine:didFailVehicleUpdate:withError:
 * to confirm success or failure. Failures are retried automatically if
 * @c locationTrackingEnabled is set to YES.
 *
 * @param vehicleState The desired vehicle state.
 */
- (void)updateVehicleState:(GMTDVehicleState)vehicleState;

/**
 * Adds a listener. The listener is held with a weak reference.
 *
 * @param listener An object conforming to the @c GMTDVehicleReporterListener protocol.
 */
- (void)addListener:(id<GMTDVehicleReporterListener>)listener;

/**
 * Removes a listener.
 *
 * @param listener An object conforming to the @c GMTDVehicleReporterListener protocol.
 */
- (void)removeListener:(id<GMTDVehicleReporterListener>)listener;

@end

NS_ASSUME_NONNULL_END
