//
//  GMTSTask.h
//  Google Ridesharing SDK for iOS
//
//  Copyright 2021 Google LLC
//
//  Usage of this SDK is subject to the Google Maps/Google Earth APIs Terms of
//  Service: https://cloud.google.com/maps-platform/terms
//

#import <Foundation/Foundation.h>


#import "GMTDDeprecationMacros.h"
#import "GMTSFleetEngineIDString.h"
#import "GMTSImmutableData.h"

@class GMSNavigationWaypoint;

NS_ASSUME_NONNULL_BEGIN

/**
 * \defgroup GMTSTaskState GMTSTaskState
 * @{
 */
typedef NS_ENUM(NSUInteger, GMTSTaskState) {
  GMTSTaskStateUnspecified = 0,
  GMTSTaskStateOpened,
  GMTSTaskStateClosed,
};

/**@}*/

/**
 * \defgroup GMTSTaskOutcome GMTSTaskOutcome
 * @{ */
typedef NS_ENUM(NSUInteger, GMTSTaskOutcome) {
  GMTSTaskOutcomeUnspecified = 0,
  GMTSTaskOutcomeSuccess,
  GMTSTaskOutcomeFailure,
};

/**@}*/

/**
 * \defgroup GMTSTaskOutcomeLocationSource GMTSTaskOutcomeLocationSource
 * @{ */
typedef NS_ENUM(NSInteger, GMTSTaskOutcomeLocationSource) {
  GMTSTaskOutcomeLocationSourceUnspecified = 0,
  GMTSTaskOutcomeLocationSourceProvider = 2,
  GMTSTaskOutcomeLocationSourceLastVehicleLocation = 3,
};

/**@}*/

/**
 * \defgroup GMTSTaskType GMTSTaskType
 * @{ */
typedef NS_ENUM(NSUInteger, GMTSTaskType) {
  GMTSTaskTypeUnspecified = 0,
  GMTSTaskTypeDeliveryPickup,
  GMTSTaskTypeDeliveryDelivery,
  GMTSTaskTypeDeliveryScheduledStop,
  GMTSTaskTypeDeliveryUnavailable,
  GMTSTaskTypeRidesharingPickup,
  GMTSTaskTypeRidesharingDropoff,
  GMTSTaskTypeRidesharingIntermediateStop,

};

/**@}*/

/** An immutable object representing a unit of work that is completed at a vehicle stop. */
@interface GMTSTask : GMTSImmutableData

/** The ID of the task. */
@property(nonatomic, readonly, nullable) GMTSFleetEngineIDString *taskID;

/** The ID associated with the provider. */
@property(nonatomic, readonly, nullable) NSString *providerID;

/** The type of current task. */
@property(nonatomic, readonly) GMTSTaskType taskType;

/** The current execution state of the task. */
@property(nonatomic, readonly) GMTSTaskState taskState;

/** The outcome of the task. */
@property(nonatomic, readonly) GMTSTaskOutcome taskOutcome;

/** The timestamp of when the task's outcome was updated (from server).*/
@property(nonatomic, readonly) NSTimeInterval taskOutcomeTimestamp;

/** Waypoint where the task's outcome was set.*/
@property(nonatomic, readonly, nullable) GMSNavigationWaypoint *taskOutcomeWaypoint;


/** Indicates the source of the location information in the taskOutcomeWaypoint.*/
@property(nonatomic, readonly) GMTSTaskOutcomeLocationSource taskOutcomeLocationSource;

/**Lets you store an ID so that the customer can avoid unnecessary or complicated mapping.*/
@property(nonatomic, readonly, nullable) GMTSFleetEngineIDString *trackingID;

/** The ID of the vehicle performing this task. */
@property(nonatomic, readonly, nullable) GMTSFleetEngineIDString *vehicleID;

/** The waypoint where the task is to be completed. */
@property(nonatomic, readonly, nullable) GMSNavigationWaypoint *plannedWaypoint;


/**  Additional time to perform an action at this location. */
@property(nonatomic, readonly) NSTimeInterval taskDuration;

/**
 * Instantiates an instance of this class.
 *
 * @param taskID  The ID of a task.
 * @param providerID The ID associated with the provider.
 * @param taskType The type of task.
 * @param taskState The current execution state of the task.
 * @param taskOutcome The outcome of the task.
 * @param taskOutcomeTimestamp  The time when the task's outcome was set.
 * @param taskOutcomeWaypoint Waypoint where the task's outcome was set.
 * @param taskOutcomeLocationSource Indicates whether the value of the task_outcome_location came
 * from unnecessary or complicated mapping.
 * @param trackingID Lets you store an ID so that the customer can avoid an unnecessary or
 * complicated mapping.
 * @param vehicleID  The ID of the vehicle performing this Task.
 * @param plannedWaypoint  The waypoint where the task will be completed.
 * @param taskDuration  Additional time to perform an action at this location.
 */
- (instancetype)initWithTaskID:(nullable GMTSFleetEngineIDString *)taskID
                    providerID:(nullable NSString *)providerID
                      taskType:(GMTSTaskType)taskType
                     taskState:(GMTSTaskState)taskState
                   taskOutcome:(GMTSTaskOutcome)taskOutcome
          taskOutcomeTimestamp:(NSTimeInterval)taskOutcomeTimestamp
           taskOutcomeWaypoint:(nullable GMSNavigationWaypoint *)taskOutcomeWaypoint
     taskOutcomeLocationSource:(GMTSTaskOutcomeLocationSource)taskOutcomeLocationSource
                    trackingID:(nullable GMTSFleetEngineIDString *)trackingID
                     vehicleID:(nullable GMTSFleetEngineIDString *)vehicleID
               plannedWaypoint:(nullable GMSNavigationWaypoint *)plannedWaypoint
                  taskDuration:(NSTimeInterval)taskDuration NS_DESIGNATED_INITIALIZER;


/** Use the designated initializer instead. */
- (instancetype)init NS_UNAVAILABLE;

@end

NS_ASSUME_NONNULL_END
